# -*- coding: utf-8 -*-
#  Copyright (C) 2006, 2007, 2008 Stewart Adam
#  This file is part of fwfstab.

#  fwfstab is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.

#  fwfstab is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with fwfstab; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
This file interfaces with the fstab.
"""
import os, os.path
import re
import shutil
#--
import fwfstab

def read(fstabFile):
  """ Read from a fstab """
  fh = open(fstabFile, 'r')
  lines = []
  for line in fh.readlines():
    # remove leading whitespace
    # FIXME: This doesn't really solve \t\ \t or \ \t\ combos.
    # this removes leading writespace and effectively removes comments after
    # complete lines. As well, it has the handy side effect of turning real
    # comments into [''] :D
    line = line.split('#')[0].strip().lstrip(' ').lstrip('\t')
    if not line or line == '\n':
      continue
    # turn \t\t -> \t
    p = re.compile('[\t]+')
    singleTabs = p.sub('\t', line.strip('\n'))
    # turn "\ \"  -> "\ "
    p = re.compile('[ ]+')
    allTabs = p.sub('\t', singleTabs)
    # turn \\t (used to be a valid space) --> "\ "
    # \\\\ --> \\ regex --> a real \
    p = re.compile('(\\\\\t)')
    workingBackslashes = p.sub('\\\\ ', allTabs)
    allTabs = p.sub('\t', workingBackslashes)
    lines.append(allTabs.split('\t'))
  fh.close()
  return lines

def write(fstabFile, content=[]):
  """ Write to a fstab. `content' is a list containing the lines to write
      (Don't include the \n at the end)"""
  if os.path.exists(fstabFile) and fwfstab.CheckPerms('%s.bak' % fstabFile):
    # back it up
    shutil.copyfile(fstabFile, '%s.bak' % fstabFile)
  else:
    return False
  # now we can safely do our work
  fh = open(fstabFile, 'w')
  for line in content:
    # FIXME: We should do some basic error checking on the input here.
    #print line
    fh.write(line + '\n')
  fh.close()
  return True




