#! /usr/bin/python
#    Copyright (C) 2006, 2007, 2008 Stewart Adam
#    This file is part of fwfstab.

#    fwfstab is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    fwfstab is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with fwfstab; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import getopt
import gtk
import gtk.glade
import gobject
import os
import pango
import re
import sys
#--
import fwfstab
from fwfstab.const import *
from fwfstab.i18n import _
from fwfstab import fstab
from fwfstab import interface
from fwfstab import widgets

class fwfstabCallbacks(interface.Controller):
  def __init__(self, fstabFile, useAutodetect=True):
    """Initialize a new instance.
    `ui' is the user interface to be controlled."""
    # setup
    interface.Controller.__init__(self, '%s/fwfstab.glade' % os.path.dirname(__file__), 'main')
    try:
      self.fstabFile = os.path.abspath(fstabFile)
      if UID != 0:
        self.useAutodetect = False
      else:
        self.useAutodetect = useAutodetect
      # start doing stuff here
      self.setTitle(self.fstabFile)
      # transient for...
      self.ui.about.set_transient_for(self.ui.main)
      self.ui.error.set_transient_for(self.ui.main)
      self.ui.confirm.set_transient_for(self.ui.main)
      # set version in About
      self.ui.aboutVersionLabel.set_text('<span size="xx-large" weight="bold">fwfstab %s</span>' % fwfstab.__version__)
      self.ui.aboutVersionLabel.set_use_markup(True)
      # icons
      if os.path.exists(APPICON):
        try:
          self.ui.aboutAppImage.set_from_file(APPICON)
          self.ui.main.set_icon_from_file(APPICON)
          self.ui.editor.set_icon_from_file(APPICON)
        except:
          pass
      else:
        print 'Could not locate program image!'
      # Widget setup
      self.statusbar = widgets.StatusBar(self.ui.statusbar1)
      # Treeview Setup
      widgets.setupEntriesTreeview(self.ui.mainEntriesTreeview)
      widgets.setupOptionsTreeview(self.ui.editorOptionsTreeview)
      widgets.setupOptionsTreeview(self.ui.optionsTreeview)
      # drives combobox/liststore
      liststore = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
      self.ui.editorDevicePreconfiguredCombobox.set_model(liststore)
      cell = gtk.CellRendererText()
      self.ui.editorDevicePreconfiguredCombobox.clear()
      self.ui.editorDevicePreconfiguredCombobox.pack_start(cell, True)
      self.ui.editorDevicePreconfiguredCombobox.add_attribute(cell, 'text', 1)
      liststore = gtk.ListStore(gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
      self.ui.editorDevicePreconfigured1Combobox.set_model(liststore)
      cell = gtk.CellRendererText()
      self.ui.editorDevicePreconfigured1Combobox.clear()
      self.ui.editorDevicePreconfigured1Combobox.pack_start(cell, True)
      self.ui.editorDevicePreconfigured1Combobox.add_attribute(cell, 'text', 1)
      # do we autodetect drives?
      if self.useAutodetect:
        self._setDevicePreconfigured(True)
        self._setFilesystemAutomatic(True)
      else:
        self.ui.editorDeviceRadio2.set_active(True)
        self.ui.editorFilesystemRadio2.set_active(True)
        #self._setDevicePreconfigured(False)
        #self._setFilesystemAutomatic(False)
        self.ui.editorDeviceRadio1.set_sensitive(False)
        self.ui.editorFilesystemRadio1.set_sensitive(False)
      self._setOptionsAuto(True)
      # Refresh data
      self.refreshMountOptions()
      self.refreshFstab()
    except:
      widgets.reportBug()

  # WRAPPERS
  def quit(self, widget=None, event=None):
    """ Quit the program """
    if self.changes == True:
      retval = widgets.displayConfirm(self, _('Unsaved Changes'),
                                   _('There are unsaved changes. Would you like to save before quitting?'))
      if retval == gtk.RESPONSE_YES:
        self.saveChanges()
    try:
      gtk.main_quit()
    except RuntimeError, msg:
      pass
    return False

  def hide(self, widget, event=None):
    """ Used to hide a close non-destructively """
    widget.hide()
    return True

  def refreshFstab(self):
    """ Adds a fstab line to the treeview. """
    try:
      errors = False
      for entry in fstab.read(self.fstabFile):
        if len(entry) != 6:
            errors = True
            print _('ERROR: Error parsing fstab entry with length %s:' % len(entry)), entry
        else:
          try:
            self.ui.mainEntriesTreeview.get_model().append(entry)
          except ValueError:
            error = True
            print _('ERROR: Error parsing fstab entry with length %s:' % len(entry)), entry
            continue
    except IOError, errorDesc:
      widgets.displayError(self, _('Filesystem error'), str(errorDesc))
    except:
      widgets.reportBug()
    finally:
      if errors:
        widgets.displayError(self, _('Could not add all entries'), _('Some entires cannot be edited.'))

  def refreshMountOptions(self):
    """ Refreshes the mount options window with contents of OPTIONS """
    model = self.ui.optionsTreeview.get_model()
    model.clear()
    for option in OPTIONS:
      model.append([option[0], option[1]])

  def refreshDrives(self):
    """ Refreshes available disks """
    if not self.useAutodetect:
      return False
    model = self.ui.editorDevicePreconfiguredCombobox.get_model()
    model.clear()
    try:
      drives = fwfstab.getHardDrives()
      if not drives:
        print _('No drives found!')
      for drive in drives:
        print _('Drive found: %(a)s: %(b)s') % {'a': drive.devnode, 'b': drive.desc}
        model.append([drive, '%s: %s' % (drive.devnode, drive.desc)])
    except Exception, errorDesc:
      widgets.reportBug()

  def saveChanges(self):
    """ Save changes """
    def saveRow(model, path, iter, data):
      currRow = []
      for col in range(0, 6):
        currRow.append(model.get_value(iter, col))
      data.append('\t'.join(currRow))
    data = []
    self.ui.mainEntriesTreeview.get_model().foreach(saveRow, data)
    try:
      if not fstab.write(self.fstabFile, data):
        widgets.displayError(self, _('Writing failed'), _('fwfstab could not write the fstab or could not back up the current one.\nPlease make sure that `%s\' still exists and you have permissions to write to the file.' % self.fstabFile))
      self.setTitle(self.fstabFile, changes=False)
      self.statusbar.newmessage(_('Saved fstab'))
    except:
      widgets.reportBug()

  def setTitle(self, fstabFile, changes=False):
    """ Sets the window title """
    self.changes = changes
    if self.changes:
      prefix = '*'
    else:
      prefix = ''
    # shortens filename if too long
    dirName, fileName = os.path.split(self.fstabFile)
    if not dirName:
      dirName = os.getcwd()
    if len(dirName) > 40:
      dirName = '%s...%s' % (dirName[:15], dirName[-15:])
    self.ui.main.set_title('%s%s (%s) - fwfstab' % (prefix, fileName, dirName))

  def _setLocked(self, bool):
    """ (Un)Lock the UI """
    self.ui.mainVButtonBox.set_sensitive(not bool)
    self.ui.mainEntriesTreeview.set_sensitive(not bool)

  # MENUS
  ## File menu
  def on_open1_activate(self, widget):
    """ Open new fstab file """
    pathBrowser = fwfstab.PathBrowser(self.ui.chooser)
    pathBrowser.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
    pathBrowser.set_select_multiple(False)
    response = pathBrowser.run()
    path = pathBrowser.get_filename()
    pathBrowser.destroy()
    if response != gtk.RESPONSE_OK:
      return
    self.fstabFile = os.path.abspath(path[0])
    self.setTitle(self.fstabFile)
    self.refreshFstab()

  def on_save_as1_activate(self, widget):
    """ Save fstab file as... """
    pathBrowser = fwfstab.PathBrowser(self.ui.chooser)
    pathBrowser.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)
    pathBrowser.set_select_multiple(False)
    response = pathBrowser.run()
    path = pathBrowser.get_filename()
    pathBrowser.destroy()
    if response != gtk.RESPONSE_OK:
      return
    self.fstabFile = os.path.abspath(path[0])
    self.setTitle(self.fstabFile)
    self.saveChanges()

  def on_apply_changes1_activate(self, widget):
    """ Save Changes """
    self.saveChanges()

  def on_quit1_activate(self, wiget):
    """ Destroy main window, quit program """
    self.ui.main.destroy()

  ## Help menu
  def on_about1_activate(self, widget):
    """ Show about window """
    self.ui.about.show()

  def on_fstab_options1_activate(self, widget):
    """ Show options window """
    self.ui.options.show()

  # ABOUT WINDOW
  def on_aboutCloseButton_clicked(self, widget):
    """ About window Close button """
    self.ui.about.hide()

  # OPTIONS WINDOW
  def on_optionsCloseButton_clicked(self, widget):
    """ Close options window """
    self.ui.options.hide()

  # EDITOR WINDOW
  def on_editorHelpButton_clicked(self, widget):
    """ Editor Help button """
    self.ui.options.show()

  def on_editorApplyButton_clicked(self, widget):
    """ Editor OK button """
    self.closeEditor()

  def on_editorCancelButton_clicked(self, widget, event=None):
    """ Editor Close button """
    self.ui.editor.hide()
    ##self.action = None
    self._setLocked(False)

  def showEditor(self, action=None):
    """ Show detailed editor, clearing or filling in fields """
    self.action = action
    self.refreshDrives()
    self.ui.editorDevicePreconfiguredCombobox.set_active(0)
    self.ui.editorDevicePreconfigured1Combobox.set_active(0)
    self.ui.editorOptionAutoCombobox.set_active(0)
    self.ui.editorValueAutoCombobox.set_active(0)
    if self.action == None:
      widgets.displayError(self, _('Internal Error'),
                        _('Please try to add/edit the entry again.\n' + \
                          'If the problem persists, report a bug against fwfstab.'))
      return
    elif self.action == 'add':
      # clean out
      if self.useAutodetect:
        self.ui.editorDeviceRadio1.set_active(True)
      else:
        self.ui.editorDeviceRadio2.set_active(True)
      self.ui.editorDeviceManualEntry.set_text('')
      if self.useAutodetect:
        self.ui.editorFilesystemRadio1.set_active(True)
      else:
        self.ui.editorFilesystemRadio2.set_active(True)
      self.ui.editorFilesystemManualEntry.set_text('')
      self.ui.editorMountpointEntry.set_text('')
      self.ui.editorOptionsEntry.set_text('defaults')
      #self.ui.editorOptionRadio1.set_active(True)
      #self.ui.editorOptionManualEntry.set_text('')
      #self.ui.editorValueManualEntry.set_text('')
      self.ui.editorDumpFSCheck.set_active(False)
      self.ui.editorCheckFSCheck.set_active(False)
    elif self.action == 'edit':
      try:
        model, iter = self.ui.mainEntriesTreeview.get_selection().get_selected()
        node = model.get_value(iter, 0)
        mountPoint = model.get_value(iter, 1)
        fsType = model.get_value(iter, 2)
        options = model.get_value(iter, 3)
        dump = model.get_value(iter, 4)
        check = model.get_value(iter, 5)
        #p = re.compile('[0-9]$')
        #node = p.sub('', val0).split('/')[-1]
        #if fwfstab.getHardDrive(node):
        #  self.ui.editorDeviceRadio1.set_active(True)
        self.ui.editorDeviceRadio2.set_active(True)
        self.ui.editorFilesystemRadio2.set_active(True)
        self.ui.editorDeviceManualEntry.set_text(node)
        self.ui.editorFilesystemManualEntry.set_text(fsType)
        self.ui.editorMountpointEntry.set_text(mountPoint)
        self.ui.editorOptionsEntry.set_text(options)
        if dump == '0':
          self.ui.editorDumpFSCheck.set_active(False)
        else:
          self.ui.editorDumpFSCheck.set_active(True)
        # FIXME: This can be any number... We shouldn't use a check
        if check == '0':
          self.ui.editorCheckFSCheck.set_active(False)
        else:
          self.ui.editorCheckFSCheck.set_active(True)
      except TypeError:
        self.statusbar.newmessage(_('Please select a row'))
        return
    self._setLocked(True)
    self.ui.editor.show()

  def closeEditor(self):
    """ Hide Editor window, saving changes or adding new entry """
    if self.action == None:
      widgets.displayError(self, _('Internal Error'),
                        _('Please try to add/edit the entry again.\n' + \
                          'If the problem persists, report a bug against fwfstab.'))
      return False
    model, treeIter = self.ui.mainEntriesTreeview.get_selection().get_selected()
    if self.ui.editorDeviceRadio1.get_active():
      nodeModel = self.ui.editorDevicePreconfigured1Combobox.get_model()
      iter = self.ui.editorDevicePreconfigured1Combobox.get_active_iter()
      node = fwfstab.getPartitionNode(nodeModel.get_value(iter, 0))
    elif self.ui.editorDeviceRadio2.get_active():
      node = self.ui.editorDeviceManualEntry.get_text()
    if self.ui.editorFilesystemRadio1.get_active():
      fsType = self.ui.editorFilesystemAutodetectLabel.get_text()
    elif self.ui.editorFilesystemRadio2.get_active():
      fsType = self.ui.editorFilesystemManualEntry.get_text()
    mountPoint = self.ui.editorMountpointEntry.get_text()
    options = self.ui.editorOptionsEntry.get_text()
    if self.ui.editorCheckFSCheck.get_active():
      dump = '1'
    else:
      dump = '0'
    if self.ui.editorDumpFSCheck.get_active():
      check = '1'
    else:
      check = '0'
    if not node or not mountPoint or not fsType:
      widgets.displayError(self, _('Invalid Input'), _('Please fill in all the fields.'))
      return False
    # now do stuff
    model = self.ui.mainEntriesTreeview.get_model()
    if self.action == 'add':
      model.append([node, mountPoint, fsType, options, dump, check])
      self.setTitle(self.fstabFile, changes=True)
      self.statusbar.newmessage(_('Added entry `%s\''))
    elif self.action == 'edit':
      #edit = model.get_value(treeIter, 0)
      model.set_value(treeIter, 0, node)
      model.set_value(treeIter, 1, mountPoint)
      model.set_value(treeIter, 2, fsType)
      model.set_value(treeIter, 3, options)
      model.set_value(treeIter, 4, dump)
      model.set_value(treeIter, 5, check)
      self.setTitle(self.fstabFile, changes=True)
      self.statusbar.newmessage(_('Modified entry `%s\'' % node))
    else:
      widgets.displayError(self, _('Internal Error'), _('self.action has a value of `%s\' which is unhandled.' % self.action))
    self.ui.editor.hide()
    self.action = None
    self._setLocked(False)
    return True

  # MAIN WINDOW
  def on_mainNewEntryButton_clicked(self, widget):
    """ Main window Add button """
    self.showEditor('add')

  def on_mainEditEntryButton_clicked(self, widget):
    """ Main window Edit button """
    self.showEditor('edit')

  def on_mainEntriesTreeview_row_activated(self, widget, path, column):
    """ User double-clicked on an entry """
    self.on_mainEditEntryButton_clicked(None)

  # Remove
  def on_mainDeleteEntryButton_clicked(self, widget):
    model, iterator = self.ui.mainEntriesTreeview.get_selection().get_selected()
    try:
      drive = model.get_value(iterator, 0)
      path = model.get_value(iterator, 1)
      self.statusbar.newmessage(_('Removed entry `%s\'' % drive))
      model.remove(iterator)
    except TypeError:
      self.statusbar.newmessage(_('Please select a row'))

  def on_mainMountButton_clicked(self, widget):
    """ Mounts the selected drive """
    if self.changes == True:
      retval = widgets.displayConfirm(self, _('Unsaved Changes'),
                                   _('(Un)mounting a drive may not work until changes are applied.\nWould you like to save now?'))
      if retval == gtk.RESPONSE_YES:
        self.saveChanges()
    model, iterator = self.ui.mainEntriesTreeview.get_selection().get_selected()
    try:
      drive = model.get_value(iterator, 0)
      path = model.get_value(iterator, 1)
      self.statusbar.newmessage(_('Mounted drive `%(a)s\' with mountpoint `%(b)s\'' % {'a': drive, 'b': path}))
    except TypeError:
      self.statusbar.newmessage(_('Please select a row'))
      return
    self.cmd_errout, self.cmd_in = os.popen4("mount " + path)
    text = []
    for i in self.cmd_in.readlines():
        # if the split has any content, do:
      if len(i) > 0:
        text.append(i)
    error = '\n'.join(text).strip('\n')
    if error != '':
      widgets.displayError(self, _('Error mounting drive'), error)
    self.cmd_errout.close()
    self.cmd_in.close()

  def on_mainUnmountButton_clicked(self, widget):
    """ Unmounts the selected drive """
    if self.changes == True:
      retval = widgets.displayConfirm(self, _('Unsaved Changes'),
                                   _('(Un)mounting a drive may not work until changes are applied. Would you like to save now?'))
      if retval == gtk.RESPONSE_YES:
        self.saveChanges()
    model, iterator = self.ui.mainEntriesTreeview.get_selection().get_selected()
    try:
      drive = model.get_value(iterator, 0)
      path = model.get_value(iterator, 1)
      self.statusbar.newmessage(_('Unmounted drive `%(a)s\' with mountpoint `%(b)s\'' % {'a': drive, 'b': path}))
    except TypeError:
      self.statusbar.newmessage(_('Please select a row'))
      return
    self.cmd_errout, self.cmd_in = os.popen4("umount " + path)
    text = []
    for i in self.cmd_in.readlines():
        # if the split has any content, do:
      if len(i) > 0:
        text.append(i)
    error = '\n'.join(text).strip('\n')
    if error != '':
      widgets.displayError(self, _('Error unmounting drive'), error)
    self.cmd_errout.close()
    self.cmd_in.close()

  def _setDevicePreconfigured(self, bool):
    """ ^ Set the UI accordingly """
    for i in ['editorDevicePreconfiguredCombobox',
              'label71',
              'editorDevicePreconfigured1Combobox']:
      getattr(self.ui, i).set_sensitive(bool)
    self.ui.editorDeviceManualEntry.set_sensitive(not bool)

  def _setFilesystemAutomatic(self, bool):
    """ ^ Set the UI accordingly """
    self.ui.editorFilesystemAutodetectLabel.set_sensitive(bool)
    self.ui.editorFilesystemManualEntry.set_sensitive(not bool)

  def _setOptionsAuto(self, bool):
    """ ^ Set the UI accordingly """
    for i in ['editorOptionAutoCombobox',
              'label65',
              'editorValueAutoCombobox']:
      getattr(self.ui, i).set_sensitive(bool)
    for i in ['editorOptionManualEntry',
              'label70',
              'editorValueManualEntry']:
      getattr(self.ui, i).set_sensitive(not bool)

  def on_editorDeviceRadio1_toggled(self, widget):
    self._setDevicePreconfigured(True)
  def on_editorDeviceRadio2_toggled(self, widget):
    self._setDevicePreconfigured(False)
  def on_editorFilesystemRadio1_toggled(self, widget):
    self._setFilesystemAutomatic(True)
  def on_editorFilesystemRadio2_toggled(self, widget):
    self._setFilesystemAutomatic(False)
  def on_editorOptionRadio1_toggled(self, widget):
    self._setOptionsAuto(True)
  def on_editorOptionRadio2_toggled(self, widget):
    self._setOptionsAuto(False)

  def on_editorDevicePreconfiguredCombobox_changed(self, widget):
    """ Drive selected: Populate xxx: drive.desc entries into combobox """
    model = self.ui.editorDevicePreconfiguredCombobox.get_model()
    active = self.ui.editorDevicePreconfiguredCombobox.get_active_iter()
    if not active:
      return
    drive = model.get_value(active, 0)
    #print 'node', node
    #drive = fwfstab.getHardDrive(node)
    partModel = self.ui.editorDevicePreconfigured1Combobox.get_model()
    partModel.clear()
    partitions = drive.getPartitions()
    if not partitions:
      widgets.displayError(self, _('Could not locate partitions'),
                        _('fwfstab could not autodetect partitions on the selected drive.'))
    print _('Using drive: %(a)s: %(b)s') % {'a': drive.devnode, 'b': drive.desc}
    for partition in partitions:
      print _('--> Partition found: %(a)s, %(b)s, %(c)sMB') % {'a': fwfstab.getPartitionNode(partition),
                                                               'b': fwfstab.getPartitionType(partition),
                                                               'c': drive.getPartitionLength(partition)}
      size = drive.getPartitionLength(partition)
      suffix = _('MB')
      if size >= 1024:
        size = round(size/1024, 2)
        suffix = _('GB')
      partModel.append([partition, '%s (%s%s)' % (fwfstab.getPartitionNode(partition),
                                                  size, suffix)
                      ])
    self.ui.editorDevicePreconfigured1Combobox.set_active(0)
    
  def on_editorDevicePreconfigured1Combobox_changed(self, widget):
    """ Partition selected: Populate /dev/xxx (XX.XXMB|GB) entries into combobox """
    #model = self.ui.editorDevicePreconfiguredCombobox.get_model()
    #active = self.ui.editorDevicePreconfiguredCombobox.get_active_iter()
    #drive = model.get_value(active, 0)
    model = self.ui.editorDevicePreconfigured1Combobox.get_model()
    active = self.ui.editorDevicePreconfigured1Combobox.get_active_iter()
    if active:
      partition = model.get_value(active, 0)
      # manual
      self.ui.editorDeviceManualEntry.set_text(fwfstab.getPartitionNode(partition))
      # part type
      self.ui.editorFilesystemAutodetectLabel.set_text(fwfstab.getPartitionType(partition))

  def on_editorMountPointBrowseButton_clicked(self, widget):
    """ Browse for a directory and fill in the path """
    pathBrowser = fwfstab.PathBrowser(self.ui.chooser)
    pathBrowser.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
    pathBrowser.set_select_multiple(False)
    response = pathBrowser.run()
    path = pathBrowser.get_filename()
    pathBrowser.destroy()
    if response != gtk.RESPONSE_OK:
      return
    self.ui.editorMountpointEntry.set_text(path[0])

def usage():
    """ Print the application usage """
    print _("""Usage:\tfwfstab [OPTIONS] [FILE]
Valid Options:
\t-h, --help\n\t\tPrint this message and exit
\t-a, --autodetect\n\t\tTry to autodetect drives (default, root privledges required)
\t-n, --no-autodetect\n\t\tDon't try to autodetect drives
FILE is assummed to be /etc/fstab when omitted.""")

# Only if we're in main execution
if __name__ == "__main__":
  autodetect = True
  try:
    avalableOptions = ["help", "autodetect", "no-autodetect"]
    # letter = plain options
    # letter: = option with arg
    (opts, rest_args) = getopt.getopt(sys.argv[1:],"han", avalableOptions)
  except (getopt.GetoptError), errorDesc:
    print errorDesc
    usage()
    sys.exit(1)
  # Remove options from file
  fstabFile = sys.argv[1:]
  for i in opts:
    for ii in i:
      try:
        fstabFile.remove(ii)
      except:
          pass
  if not fstabFile:
    fstabFile = ['/etc/fstab']
  # Parse args, take action
  if opts:
    for (opt, value) in opts:
      if opt == "-h" or opt == "--help":
        usage()
        sys.exit(1)
      elif opt == "-a" or opt == "--autodetect":
        autodetect = True
      elif opt == "-n" or opt == "--no-autodetect":
        autodetect = False
      #if opt == "-X" or opt == "--X":
      #   dostuffhere(value)

# Startup the application and call the gtk event loop
if __name__ == "__main__":
  try:
    mainApp = fwfstabCallbacks(fstabFile[0], autodetect)
    gtk.main()
  except KeyboardInterrupt:
    # ctrl+c?
    mainApp.quit(None)

